@@ -9,9 +9,10 @@
 
 import os
 import sys
+from typing import Any
 from time import monotonic
 from threading import RLock
-from typing import Any
+from collections.abc import Sequence
 
 from xpra.codecs.constants import TransientCodecException
 from xpra.util.objects import typedict
@@ -24,7 +25,7 @@ from xpra.platform.paths import (
 from xpra.os_util import WIN32
 from xpra.util.system import is_WSL
 from xpra.util.io import load_binary_file
-from xpra.log import Logger
+from xpra.log import Logger, consume_verbose_argv
 
 log = Logger("cuda")
 
@@ -34,7 +35,7 @@ if WIN32 and not os.environ.get("CUDA_PATH") and getattr(sys, "frozen", None) in
 if is_WSL() and not envbool("XPRA_PYCUDA_WSL", False):
     raise ImportError("refusing to import pycuda on WSL, use `XPRA_PYCUDA_WSL=1` to override")
 
-with numpy_import_context("CUDA"):
+with numpy_import_context("CUDA context import"):
     import pycuda
     log(f"loaded pycuda successfully: {pycuda}")
     from pycuda.driver import (
@@ -81,7 +82,7 @@ def compute_capability(d) -> int:
     return (smmajor << 4) + smminor
 
 
-def get_pycuda_version() -> tuple[int, ...]:
+def get_pycuda_version() -> Sequence[int]:
     return pycuda.VERSION
 
 
@@ -348,7 +349,7 @@ def reset_state() -> None:
     DEVICE_STATE = {}
 
 
-def select_device(preferred_device_id=-1, min_compute=0):
+def select_device(preferred_device_id=-1, min_compute=0) -> tuple[int, Any]:
     log("select_device(%s, %s)", preferred_device_id, min_compute)
     for device_id in (preferred_device_id, get_pref("device-id")):
         if device_id is not None and device_id >= 0:
@@ -376,7 +377,7 @@ def select_device(preferred_device_id=-1, min_compute=0):
 rr = 0
 
 
-def select_round_robin(min_compute: int):
+def select_round_robin(min_compute: int) -> tuple[int, Any]:
     if not driver_init():
         return -1, None
     enabled_gpus = get_gpu_list("enabled-devices")
@@ -441,10 +442,8 @@ def select_best_free_memory(min_compute: int = 0) -> tuple[int, Any]:
                     context.pop()
                     context.detach()
         if selected_device_id >= 0 and selected_device:
-            l = log.debug
-            if len(devices) > 1:
-                l = log.info
-            l("selected device %s: %s", selected_device_id, device_info(selected_device))
+            log_fn = log.info if len(devices) > 1 else log.debug
+            log_fn("selected device %s: %s", selected_device_id, device_info(selected_device))
             return selected_device_id, selected_device
     return -1, None
 
@@ -510,7 +509,7 @@ def get_default_device_context():
 class cuda_device_context:
     __slots__ = ("device_id", "device", "context", "lock", "opengl")
 
-    def __init__(self, device_id, device, opengl=False):
+    def __init__(self, device_id: int, device, opengl=False):
         assert device, "no cuda device"
         self.device_id = device_id
         self.device = device
@@ -532,7 +531,7 @@ class cuda_device_context:
     def make_context(self) -> None:
         start = monotonic()
         if self.opengl:
-            with numpy_import_context("CUDA"):
+            with numpy_import_context("CUDA make context"):
                 from pycuda import gl  # @UnresolvedImport pylint: disable=import-outside-toplevel
                 self.context = gl.make_context(self.device)
         else:
@@ -581,7 +580,8 @@ class cuda_device_context:
 
     def free(self) -> None:
         c = self.context
-        log("free() context=%s", c)
+        if log:
+            log("free() context=%s", c)
         if c:
             self.device_id = 0
             self.device = None
@@ -635,11 +635,9 @@ def get_CUDA_function(function_name):
 
 def main():
     # pylint: disable=import-outside-toplevel
-    if "-v" in sys.argv or "--verbose" in sys.argv:
-        log.enable_debug()
-
     from xpra.platform import program_context
     with program_context("CUDA-Info", "CUDA Info"):
+        consume_verbose_argv(sys.argv, "cuda")
         pycuda_info = get_pycuda_info()
         log.info("pycuda_info")
         print_nested_dict(pycuda_info, print_fn=log.info)
